@@ -16,7 +16,7 @@ limitations under the License.
 """
 
 import json
-from .serviceconnector import ServiceConnector
+from .serviceconnector import _Client, ServiceConnector
 from .camel import CamelResource
 from .utils import get_logger
 from .utils import raise_for_status_with_detail
@@ -24,20 +24,21 @@ from .utils import raise_for_status_with_detail
 log = get_logger(__name__)
 
 
-class ConnectionClient:
+class ConnectionClient(_Client):
     """
     A client used to manage connections.
     """
-    URIs = {'connections': 'connections'}
+    URIs = {'connections': 'projects/{projectId}/connections'}
 
-    def __init__(self, url, version, token):
-        self._serviceconnector = ServiceConnector(url, version, token)
+    def __init__(self, *args, **kwargs):
+        super().__init__(*args, **kwargs)
+        self._serviceconnector.version = 4
 
-    def save_connection(self, connection: object):
+    def save_connection(self, project: str, connection: object):
         """
         Posts the connection client information.
         """
-        uri  = self.URIs['connections']
+        uri = self.URIs['connections'].format(projectId=project)
         data = json.dumps(connection)
         headers = {'Content-Type': 'application/json'}
         r = self._serviceconnector.request('POST', uri, data, headers)
@@ -63,18 +64,20 @@ class Connection(CamelResource):
         super().__init__(connection, True)
         self._connector = connector
 
+
     @staticmethod
-    def get_connection(name, client: ConnectionClient):
+    def get_connection(name, project, client: ConnectionClient):
         """
         Fetches a Connection to work with.
 
         :param client: The client instance to use.
         :param name: The name of the connection to retrieve.
+        :param project: The project from which connection has to be retrieved.
         :return: A Connection object.
         """
-        uri = 'connections/{name}'.format(name=name)
+        uri = 'projects/{projectId}/connections/{name}'.format(projectId=project, name=name)
         log.debug('Getting connection using URI: %s' % uri)
         r = client._serviceconnector.request('GET', uri)
         raise_for_status_with_detail(r)
 
-        return Connection(r.json(), client._serviceconnector)
+        return r.json()
